package com.fjhx.tenant.controller.tenant;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.TenantIgnore;
import com.ruoyi.common.constant.BaseSourceConstant;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysPostService;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Slf4j
@TenantIgnore
@DS(BaseSourceConstant.BASE)
@RestController
@RequestMapping("/tenantUser")
public class UserTenantController extends BaseController {

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private ISysPostService postService;
    @Autowired
    private ISysDeptService sysDeptService;

    /**
     * 获取用户列表
     */
    @GetMapping("/list")
    public TableDataInfo list(SysUser user) {
        startPage();
        List<SysUser> list = userService.selectUserList(user);
        return getDataTable(list);
    }

    /**
     * 新增用户
     */
    @RepeatSubmit
    @PostMapping
    public void add(@Validated @RequestBody SysUser user) {
        String tenantId = user.getTenantId();

//        //如果不是超管赋值登录用户所在租户
//        SysUser loginUser = SecurityUtils.getLoginUser().getUser();
//        if (loginUser.getUserId() != 1) {
//            tenantId = loginUser.getTenantId();
//        }

        if (ObjectUtil.isEmpty(tenantId)) {
            throw new ServiceException("租户id不能为空");
        }

        if (!userService.checkUserNameUnique(user)) {
            throw new ServiceException("新增用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        user.setCreateBy(getUsername());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));

        //赋值归属公司
        user.setCompanyId(getCompanyByDeptId(user.getDeptId()).getDeptId());

        userService.insertUser(user);
    }

    /**
     * 修改用户
     */
    @PutMapping
    public void edit(@Validated @RequestBody SysUser user) {
        String tenantId = user.getTenantId();
//        //如果不是超管赋值登录用户所在租户
//        SysUser loginUser = SecurityUtils.getLoginUser().getUser();
//        if (loginUser.getUserId() != 1) {
//            tenantId = loginUser.getTenantId();
//        }

        if (ObjectUtil.isEmpty(tenantId)) {
            throw new ServiceException("租户id不能为空");
        }

        userService.checkUserAllowed(user);
        userService.checkUserDataScope(user.getUserId());
        if (!userService.checkUserNameUnique(user)) {
            throw new ServiceException("修改用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        user.setUpdateBy(getUsername());

        String password = user.getPassword();
        if (StrUtil.isNotBlank(password)) {
            user.setPassword(SecurityUtils.encryptPassword(password));
        }

        //赋值归属公司
        user.setCompanyId(getCompanyByDeptId(user.getDeptId()).getDeptId());

        userService.updateUser(user);
    }

    private SysDept getCompanyByDeptId(Long deptId) {
        SysDept dept = sysDeptService.getById(deptId);
        if (ObjectUtil.isEmpty(dept)) {
            throw new ServiceException("查询不到该部门公司信息");
        }
        if (Objects.equals(dept.getType(), 0)) {
            return dept;
        }
        return getCompanyByDeptId(dept.getParentId());
    }

    /**
     * 修改用户业务员编号
     */
    @PostMapping("/editUserCode")
    public void editUserCode(@RequestBody SysUser user) {
        userService.editUserCode(user);
    }

    /**
     * 删除用户
     */
    @DeleteMapping("/{userIds}")
    public AjaxResult remove(@PathVariable Long[] userIds) {
        if (ArrayUtils.contains(userIds, getUserId())) {
            return error("当前用户不能删除");
        }
        return toAjax(userService.deleteUserByIds(userIds));
    }

    /**
     * 重置密码
     */
    @PutMapping("/resetPwd")
    public void resetPwd(@RequestBody SysUser user) {
        String tenantId = SecurityUtils.getTenantId();
        Long userId = SecurityUtils.getUserId();
        log.error("调用方法：resetPwd，tenantId：{}，userId：{}，date：{}，data；{}",
                tenantId, userId, DateUtil.formatDateTime(new Date()), JSONObject.toJSONString(user));
        userService.checkUserAllowed(user);
        userService.checkUserDataScope(user.getUserId());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        user.setUpdateBy(getUsername());
        userService.resetPwd(user);
    }

    /**
     * 根据用户编号获取详细信息
     */
    @GetMapping(value = {"/", "/{userId}"})
    public AjaxResult getInfo(@PathVariable(value = "userId", required = false) Long userId) {
        userService.checkUserDataScope(userId);
        AjaxResult ajax = AjaxResult.success();
        List<SysRole> roles = roleService.selectRoleAll();
        ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect(Collectors.toList()));
        ajax.put("posts", postService.selectPostAll());
        if (StringUtils.isNotNull(userId)) {
            SysUser sysUser = userService.selectUserById(userId);
            ajax.put(AjaxResult.DATA_TAG, sysUser);
            ajax.put("postIds", postService.selectPostListByUserId(userId));
            ajax.put("roleIds", sysUser.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));
        }
        return ajax;
    }

    /**
     * 根据角色id列表获取用户分组列表
     */
    @PostMapping("/getUserListByRoleIds")
    public Map<Long, List<SysUser>> getUserListByRoleIds(@RequestBody List<Long> roleIds) {
        return userService.getUserListByRoleIds(roleIds);
    }

}
